import java.net.URISyntaxException;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;
import org.eclipse.emf.henshin.interpreter.EGraph;
import org.eclipse.emf.henshin.interpreter.Engine;
import org.eclipse.emf.henshin.interpreter.UnitApplication;
import org.eclipse.emf.henshin.interpreter.impl.EGraphImpl;
import org.eclipse.emf.henshin.interpreter.impl.EngineImpl;
import org.eclipse.emf.henshin.interpreter.impl.LoggingApplicationMonitor;
import org.eclipse.emf.henshin.interpreter.impl.UnitApplicationImpl;
import org.eclipse.emf.henshin.model.Module;
import org.eclipse.emf.henshin.model.Unit;
import org.eclipse.emf.henshin.model.resource.HenshinResourceSet;

/**
 */
public class HenshinTransformation {

	public static final LoggingApplicationMonitor log = new LoggingApplicationMonitor();

	public static void main(String[] arg0) throws URISyntaxException {

		System.out.println("Starting No-Multi-Rule Transformation");
		executeTransformation("target-no-multi-rule.xmi",
				"father_no_multi_rule");
		System.out.println("No-Multi-Rule was successful.");

		System.out.println("Starting Multi-Rule Transformation");
		executeTransformation("target-multi-rule.xmi", "father_multi_rule");
		System.out.println("Multi-Rule was successful.");
	}

	private static void executeTransformation(String targetFileName,
			String unitName) {
		HenshinResourceSet resourceSet = new HenshinResourceSet();
		resourceSet.getResourceFactoryRegistry().getExtensionToFactoryMap()
				.put("core", new XMIResourceFactoryImpl());

		// Load the example model into an EGraph:
		EGraph graph = new EGraphImpl();
		graph.addTree(resourceSet.getEObject("Family.xmi"));

		// Create an engine and a unit application:
		Engine engine = new EngineImpl();
		UnitApplication app = new UnitApplicationImpl(engine);
		app.setEGraph(graph);

		Module module = resourceSet.getModule("default.henshin", false);

		Unit unit = module.getUnit(unitName);
		app.setUnit(unit);

		if (!app.execute(log)) {
			throw new RuntimeException("Error executing rule");
		}

		// Store the result
		EObject result = (EObject) app.getResultParameterValue("ret");
		resourceSet.saveEObject(result, targetFileName);
	}

};